/*
    This file is part of Magnum.

    Copyright © 2010, 2011, 2012, 2013, 2014, 2015
              Vladimír Vondruš <mosra@centrum.cz>

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

#include "Magnum/Shaders/Flat.h"
#include "Magnum/Test/AbstractOpenGLTester.h"

#ifdef MAGNUM_BUILD_STATIC
#include "Magnum/Shaders/resourceImport.hpp"
#endif

namespace Magnum { namespace Shaders { namespace Test {

struct FlatGLTest: Magnum::Test::AbstractOpenGLTester {
    explicit FlatGLTest();

    void compile2D();
    void compile3D();
    void compile2DTextured();
    void compile3DTextured();
};

FlatGLTest::FlatGLTest() {
    addTests({&FlatGLTest::compile2D,
              &FlatGLTest::compile3D,
              &FlatGLTest::compile2DTextured,
              &FlatGLTest::compile3DTextured});
}

void FlatGLTest::compile2D() {
    Shaders::Flat2D shader;
    CORRADE_VERIFY(shader.validate().first);
}

void FlatGLTest::compile3D() {
    Shaders::Flat3D shader;
    CORRADE_VERIFY(shader.validate().first);
}

void FlatGLTest::compile2DTextured() {
    Shaders::Flat2D shader(Shaders::Flat2D::Flag::Textured);
    CORRADE_VERIFY(shader.validate().first);
}

void FlatGLTest::compile3DTextured() {
    Shaders::Flat3D shader(Shaders::Flat3D::Flag::Textured);
    CORRADE_VERIFY(shader.validate().first);
}

}}}

CORRADE_TEST_MAIN(Magnum::Shaders::Test::FlatGLTest)
